=begin
= strscan.rb

Ruby script version of Aoki's ext library strscan (for RDtool only).

NOTE: this library ((*CANNOT*)) be used universally.

=end
# '


class StringScanner
  if RUBY_VERSION > "1.5"
    REGEXP_MULTI_OPTION = "m"
  else
    REGEXP_MULTI_OPTION = "p"
  end
  
  SWAP_CONSTS = <<"RUBY"
  EM_OPEN_RE = /\\A\#{Regexp.quote(EM_OPEN)}/#{REGEXP_MULTI_OPTION}
  EM_CLOSE_RE = /\\A\#{Regexp.quote(EM_CLOSE)}/#{REGEXP_MULTI_OPTION}
  CODE_OPEN_RE = /\\A\#{Regexp.quote(CODE_OPEN)}/#{REGEXP_MULTI_OPTION}
  CODE_CLOSE_RE = /\\A\#{Regexp.quote(CODE_CLOSE)}/#{REGEXP_MULTI_OPTION}
  VAR_OPEN_RE = /\\A\#{Regexp.quote(VAR_OPEN)}/#{REGEXP_MULTI_OPTION}
  VAR_CLOSE_RE = /\\A\#{Regexp.quote(VAR_CLOSE)}/#{REGEXP_MULTI_OPTION}
  KBD_OPEN_RE = /\\A\#{Regexp.quote(KBD_OPEN)}/#{REGEXP_MULTI_OPTION}
  KBD_CLOSE_RE = /\\A\#{Regexp.quote(KBD_CLOSE)}/#{REGEXP_MULTI_OPTION}
  INDEX_OPEN_RE = /\\A\#{Regexp.quote(INDEX_OPEN)}/#{REGEXP_MULTI_OPTION}
  INDEX_CLOSE_RE = /\\A\#{Regexp.quote(INDEX_CLOSE)}/#{REGEXP_MULTI_OPTION}
  REF_OPEN_RE = /\\A\#{Regexp.quote(REF_OPEN)}/#{REGEXP_MULTI_OPTION}
  REF_CLOSE_RE = /\\A\#{Regexp.quote(REF_CLOSE)}/#{REGEXP_MULTI_OPTION}
  FOOTNOTE_OPEN_RE = /\\A\#{Regexp.quote(FOOTNOTE_OPEN)}/#{REGEXP_MULTI_OPTION}
  FOOTNOTE_CLOSE_RE = /\\A\#{Regexp.quote(FOOTNOTE_CLOSE)}/#{REGEXP_MULTI_OPTION}
  VERB_OPEN_RE = /\\A\#{Regexp.quote(VERB_OPEN)}/#{REGEXP_MULTI_OPTION}
  VERB_CLOSE_RE = /\\A\#{Regexp.quote(VERB_CLOSE)}/#{REGEXP_MULTI_OPTION}

  BAR_RE = /\\A\#{Regexp.quote(BAR)}/#{REGEXP_MULTI_OPTION}
  QUOTE_RE = /\\A\#{Regexp.quote(QUOTE)}/#{REGEXP_MULTI_OPTION}
  SLASH_RE = /\\A\#{Regexp.quote(SLASH)}/#{REGEXP_MULTI_OPTION}
  BACK_SLASH_RE = /\\A\#{Regexp.quote(BACK_SLASH)}/#{REGEXP_MULTI_OPTION}
  URL_RE = /\\A\#{Regexp.quote(URL)}/#{REGEXP_MULTI_OPTION}

  # Workaround for Regexp option change of Ruby 1.5.x
  other_re_mode = Regexp::EXTENDED
  if RUBY_VERSION > "1.5"
    other_re_mode |= Regexp::MULTILINE
  else
    other_re_mode |= Regexp::POSIXLINE
  end

  OTHER_RE = Regexp.new(
		  "\\\\A.+?(?=\#{Regexp.quote(EM_OPEN)}|\#{Regexp.quote(EM_CLOSE)}|
                  \#{Regexp.quote(CODE_OPEN)}|\#{Regexp.quote(CODE_CLOSE)}|
                  \#{Regexp.quote(VAR_OPEN)}|\#{Regexp.quote(VAR_CLOSE)}|
                  \#{Regexp.quote(KBD_OPEN)}|\#{Regexp.quote(KBD_CLOSE)}|
                  \#{Regexp.quote(INDEX_OPEN)}|\#{Regexp.quote(INDEX_CLOSE)}|
                  \#{Regexp.quote(REF_OPEN)}|\#{Regexp.quote(REF_CLOSE)}|
                \#{Regexp.quote(FOOTNOTE_OPEN)}|\#{Regexp.quote(FOOTNOTE_CLOSE)}|
                  \#{Regexp.quote(VERB_OPEN)}|\#{Regexp.quote(VERB_CLOSE)}|
                  \#{Regexp.quote(BAR)}|
                  \#{Regexp.quote(QUOTE)}|
                  \#{Regexp.quote(SLASH)}|
                  \#{Regexp.quote(BACK_SLASH)}|
                  \#{Regexp.quote(URL)})", other_re_mode)
RUBY

  EVALED_ALREADY = []

  def initialize(src)
    if EVALED_ALREADY.empty?
      RD::RDInlineParser.module_eval(SWAP_CONSTS)
#      RD::RDInlineParser.constants.each do |i|
#	STDERR.print i + " = " + RD::RDInlineParser.const_get(i).inspect + "\n"
#      end
      EVALED_ALREADY.push(true)
    end
    @src = src
  end

  def scan(re)
    if re =~ @src
      @src = $' #'
#      STDERR.print "scan: " + $& + "\n"
      $&
    else
      nil
    end
  end

  def rest
#    STDERR.print "rest: " + @src + "\n"
    @src
  end

  def clear
    @src = ""
  end

  def empty?
    @src == ""
  end  
end
